import tensorflow as tf
from tensorboard.plugins.hparams import api as hp_board
from kerastuner.engine.base_tuner import BaseTuner
import kerastuner.engine.hyperparameters as hp

from kerastuner_tensorboard_logger import TensorBoardLogger


def setup_tb(tuner: BaseTuner):
    """setup hparams"""
    if not isinstance(tuner.logger, TensorBoardLogger):
        raise ValueError("Must set TensorBoardLogger")

    hps = tuner.oracle.get_space()

    hparams = [kerastuner_to_hparams(hp) for hp in hps.space]

    metrics = tuner.logger.metrics
    hmetrics = [hp_board.Metric(metric) for metric in metrics]

    logdir = tuner.logger.logdir
    with tf.summary.create_file_writer(logdir).as_default():
        hp_board.hparams_config(
            hparams=hparams,
            metrics=hmetrics,
        )


def kerastuner_to_hparams(value: hp.HyperParameter) -> hp_board.HParam:
    """convert kerastuner hp to tensorboard hp"""
    SHORTCUT = {
        hp.Int: int_to_Hparam,
        hp.Float: float_to_Hparam,
        hp.Choice: choice_to_Hparam,
        hp.Boolean: boolean_to_Hparam,
    }
    hparam = SHORTCUT.get(type(value), to_hparam)(value)
    return hparam


def to_hparam(value: hp.HyperParameter) -> hp_board.HParam:
    """base convertor."""
    name = value.name
    return hp_board.HParam(name)


def choice_to_Hparam(value: hp.Choice) -> hp_board.HParam:
    """Choice to hp_board.HParam"""
    name = value.name
    choices = value.values
    return hp_board.HParam(name, hp_board.Discrete(choices))


def int_to_Hparam(value: hp.Int) -> hp_board.HParam:
    """Int to hp_board.Hparam"""
    name = value.name
    min_value = value.min_value
    max_value = value.max_value
    return hp_board.HParam(
        name, hp_board.IntInterval(min_value=min_value, max_value=max_value)
    )


def float_to_Hparam(value: hp.Float) -> hp_board.HParam:
    """Float to hp_board.Hparam"""
    name = value.name
    min_value = value.min_value
    max_value = value.max_value
    return hp_board.HParam(
        name, hp_board.RealInterval(min_value=min_value, max_value=max_value)
    )


def boolean_to_Hparam(value: hp.Float) -> hp_board.HParam:
    """boolean to hp_board.Hparam"""
    name = value.name
    hp_board.Discrete
    return hp_board.HParam(name, hp_board.Discrete((True, False)))



import os
from typing import Any, Dict, Generator, List, Tuple, Union
from datetime import timedelta, datetime
import tensorflow as tf
from kerastuner.engine.logger import Logger
from tensorboard.plugins.hparams import api as hp_board


def timedelta_to_hms(timedelta: timedelta) -> str:
    """convert datetime.timedelta to string like '01h:15m:30s'"""
    tot_seconds = int(timedelta.total_seconds())
    hours = tot_seconds // 3600
    minutes = (tot_seconds % 3600) // 60
    seconds = tot_seconds % 60
    return f"{hours}h:{minutes}m:{seconds}s"


class TensorBoardLogger(Logger):
    """tensorboard.plugins.hparams logger class.

    Create logger instance for tuner instance that inherit kerastuner.engine.base_tuner.BaseTuner.

    # Arguments:
        metrics: String or list of String, reports values for Tensorboard.
        logdir: String. Path to the log directory (relative) to be accessed by Tensorboard.
        overwrite: Bool, default `False`. If `False`, reloads an existing log directory
            of the same name if one is found. Otherwise, overwrites the log.
    """

    def __init__(
        self,
        metrics: Union[str, List[str]] = ["acc"],
        logdir: str = "logs/hparam_tuning",
        overwrite: bool = False,
    ):
        self.metrics = [metrics] if isinstance(metrics, str) else metrics
        self.logdir = logdir
        self.times: Dict[str, datetime] = dict()

        if overwrite and tf.io.gfile.exists(self.logdir):
            tf.io.gfile.rmtree(self.logdir)

    def register_tuner(self, tuner_state):
        """Informs the logger that a new search is starting."""
        pass

    def register_trial(self, trial_id: str, trial_state: Dict[str, Any]):
        """Informs the logger that a new Trial is starting."""
        self.times[trial_id] = datetime.now()

    def report_trial_state(self, trial_id: str, trial_state: Dict[str, Any]):
        """Gives the logger information about trial status."""
        execution_time = timedelta_to_hms(datetime.now() - self.times.pop(trial_id))
        logdir = os.path.join(self.logdir, f"{execution_time}-{trial_id}")
        with tf.summary.create_file_writer(logdir).as_default():
            hparams = self.parse_hparams(trial_state)
            hp_board.hparams(hparams)  # record the values used in this trial

            for target_metric, metric in self.parse_metrics(trial_state):
                tf.summary.scalar(target_metric, metric, step=1)

    def exit(self):
        pass

    def parse_hparams(self, trial_state: Dict[str, Any]) -> Dict[str, Any]:
        """use in tf.summary writer context"""
        hparams = trial_state.get("hyperparameters", {}).get("values", {})
        return hparams

    def parse_metrics(
        self, trial_state: Dict[str, Any]
    ) -> Generator[Tuple[str, Union[int, float]], None, None]:
        """use in tf.summary writer context"""
        metrics = trial_state.get("metrics", {}).get("metrics", {})
        for target_metric in self.metrics:
            metric = metrics.get(target_metric)
            if metric is None:
                continue
            metric = metric.get("observations", [{}])[0].get("value")
            if metric is None:
                continue
            yield target_metric, metric[0]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kerastuner_tensorboard_logger']

package_data = \
{'': ['*']}

install_requires = \
['keras-tuner>=1.0,<2.0', 'tensorflow>=2.0,<3.0']

setup_kwargs = {
    'name': 'kerastuner-tensorboard-logger',
    'version': '0.2.0',
    'description': 'Simple integration of keras-tuner (hyperparameter tuning) and tensorboard dashboard (interactive visualization).',
    'long_description': '# Keras-tuner Tensorboard logger\n\n![](https://github.com/tokusumi/kerastuner-tensorboard-logger/workflows/Tests/badge.svg)\n[![PyPI version](https://badge.fury.io/py/kerastuner-tensorboard-logger.svg)](https://badge.fury.io/py/kerastuner-tensorboard-logger)\n\n[keras-tuner](https://www.tensorflow.org/tutorials/keras/keras_tuner) logger for streaming search report to [Tensorboard plugins Hparams](https://www.tensorflow.org/tensorboard/hyperparameter_tuning_with_hparams), beautiful interactive visualization tool.\n\n## Requirements\n\n* Python 3.6+\n* keras-tuner 1.0.0+\n* Tensorboard 2.1+\n\n## Installation\n\n```\n$ pip install kerastuner-tensorboard-logger\n```\n\n## Example\n\nhere is simple (and incomplete) code.\n\nSee details about how to use keras-tuner [here](https://github.com/keras-team/keras-tuner).\n\nAdd only one argument in tuner class and search it, then you can go to see search report in Tensorboard.\n\nOptionally, you can call `setup_tb` to be more accurate TensorBoard visualization. It convert keras-tuner hyperparameter information and do [Tensorboard experimental setup](https://www.tensorflow.org/tensorboard/hyperparameter_tuning_with_hparams#1_experiment_setup_and_the_hparams_experiment_summary).\n\n```python\n# import this\nfrom kerastuner_tensorboard_logger import (\n    TensorBoardLogger,\n    setup_tb  # Optional\n)\n\ntuner = Hyperband(\n    build_model,\n    objective="val_acc",\n    max_epochs=5,\n    directory="logs/tuner",\n    project_name="tf_test",\n    logger=TensorBoardLogger(\n        metrics=["val_acc"], logdir="logs/hparams"\n    ),  # add only this argument\n)\n\nsetup_tb(tuner)  # (Optional) For more accurate visualization.\ntuner.search(x, y, epochs=5, validation_data=(val_x, val_y))\n```\n\n### Tensorboard\n\n```bash\n$ tensorboard --logdir ./logs/hparams\n```\n\nGo to http://127.0.0.1:6006.\n\nYou will see the interactive visualization (provided by Tensorboard).\n\n![Table View](https://raw.githubusercontent.com/tokusumi/kerastuner-tensorboard-logger/main/docs/src/table_view.jpg)\n\n![Parallel Coordinates View](https://raw.githubusercontent.com/tokusumi/kerastuner-tensorboard-logger/main/docs/src/parallel_coordinates_view.jpg)\n',
    'author': 'tokusumi',
    'author_email': 'tksmtoms@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tokusumi/kerastuner-tensorboard-logger',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
